/*
 * FIMPORT.C:
 * Interactive fixed-length to variable-length field conversion utility
 * Written by Larry Hansen
 * Compile/Link (Microsoft C):
 *      cl /F 1000 fimport.c
 */

#include <stdio.h>
#include <string.h>
#include <graph.h>
#include <ctype.h>
#include <stdlib.h>

void translate_fields(void);

int start[250],len[250];
int field;
FILE *in, *out;
int x;
char inbuf[2048];
char infile[80],outfile[80];

main()
{
    FILE *imp_file,*fopen();
    int line_len,k;
    char inpath[_MAX_PATH], outpath[_MAX_PATH];
    char drive[_MAX_DRIVE], dir[_MAX_DIR];
    char fname[_MAX_FNAME], ext[_MAX_EXT];

    _clearscreen(_GCLEARSCREEN);
    _settextposition(1,1);
    _outtext("Please enter the file you wish to convert:");
    gets(infile);
    _settextposition(2,1);
    _outtext("Please enter the file you wish to save to:");
    gets(outfile);

    if( (in = fopen(infile,"r")) == NULL)
    {
        printf("Could not open file\n");
        exit(1);
    }
    _splitpath( outfile, drive, dir, fname, ext );

        /* File extension to save definitions in for later use: */
    strcpy( ext, "imp" );
    _makepath( outpath, drive, dir, fname, ext );

        /* If definition file exists, then use it to break out the line: */
    if(check_file(outpath))
    {
        translate_fields();
        exit(0);
    }                          

    /* No definition file exists. Get the first line of the input file: */
    fgets(inbuf,2048,in);
    line_len = strlen(inbuf);
    if(inbuf[line_len - 1] == '\n')
        inbuf[line_len - 1] = 0;
    fclose(in);

    /* Have user assign field markers until he is happy: */

    do
    {
        _settextposition(8,1);
        _outtext("Mark fields: F / space / Q...");
        _settextposition(4, 1);
        _outtext(inbuf);
        for (x = line_len % 80; x < 80; x++)
            _outtext(" ");
        fld_break(inbuf);
        _settextposition(4, line_len);
        _settextposition(8, 1);
        _outtext("Is this information correct? ");
        k = getch();
    } while(toupper(k) != 'Y');

            /* Apply the conversion: */
    translate_fields();

            /* Save the field definitions for later re-use: */
    imp_file = fopen(outpath,"w");
    fprintf(imp_file, "%s\n", infile);
    for(x = 0; x < field; x++)
        fprintf(imp_file, "%d %d\n", start[x], len[x]);
    fclose(imp_file);
    _settextposition(10,1);
}

void translate_fields(void)
{
            /* Open the definition file */
    if( (in = fopen(infile,"r")) == NULL)
    {
        printf("Could not open file\n");
        exit(1);
    }
            /* Open the output file: */
    out = fopen(outfile,"w");

            /* read the input file a line at a time */
    while(fgets(inbuf,2048,in) != NULL)
    {
        _settextposition(4,1);
        for(x = 0; x < field - 1; x++)
        {
                    /* Print out each line: */
            printf("\"%.*s\",",len[x],&inbuf[start[x]]);
            fprintf(out,"\"%.*s\",",len[x],&inbuf[start[x]]);
        }
            /* Do the last field: */
        printf("\"%.*s\"\n",len[x],&inbuf[start[x]]);
        fprintf(out,"\"%.*s\"\n",len[x],&inbuf[start[x]]);
    }
        /* Close and get out */
    fclose(in);
    fclose(out);
}


/*
 * fld_break(char *): Let user break line up into fields interactively
 *  Takes the first line of the file to be converted as the argument.
 *  Populates the field count, start and len arrays.
 * 
 * Field marking commands:
 *  An "f" marks the current column as the last column of a field;
 *  Enter or "q" ends the marking process, registering all chars past last
 *      marked column as the final field;
 *  "bs" backspaces to the previous column.
 *  Any other char advances cursor one position to the right.
 */

fld_break(char *buf)
{
    int old_bk_color, old_fg_color;
    int new_bk_color, new_fg_color, mark_color;
    int st_count, l_count;
    int buf_index;
    int x, y;
    int max_len, k;
    char tmp[2];    /* 2-char array for Microsoft's _outtext */

    max_len = strlen(buf);
    old_bk_color = _getbkcolor();   /* Save the old colors */
    old_fg_color = _gettextcolor();

    new_bk_color = 1;           /* Blue         */
    new_fg_color = 15;          /* bright white */
    mark_color = 2;             /* green        */
    _setbkcolor((long)new_bk_color);
    _settextcolor(new_fg_color);

        /* screen positions: */
    x = 4;
    y = 1;

        /* initialize counters to 0: */
    buf_index = 0;
    st_count = 0;
    l_count = 0;
    field = 0;
    _settextposition(x,y);

    do
    {
        _settextposition(x,y);
            /* Make a string out of the individual chars in the line: */
        tmp[0] = buf[buf_index];
        tmp[1] = 0;
        k = getch();
        switch (toupper(k))
        {
            case '\b':      /* backing up a position */
                            /* decrement the arrays: */
                l_count--;
                if (l_count < 0)
                    l_count = 0;
                buf_index--;
                if (buf_index < 0)
                    buf_index = 0;
                            /* horizontal movement:     */
                y--;
                if (y < 1)
                {
                        /* more than one line: */
                    if (x > 4)
                    {
                        y = 80;
                        x--;
                    }
                        /* only one line or at start of line: */
                    else
                        y = 1;
                }
                
                _settextposition(x, y);
                _setbkcolor((long)old_bk_color);
                _settextcolor(old_fg_color);

                tmp[0] = buf[buf_index];
                tmp[1] = 0;

                    /* Handle case of backspacing over field marker: */

                if (field && (start[field - 1] + len[field - 1] - 1)
                                        == buf_index)
                {
                    field--;
                    l_count = buf_index - start[field];
                    st_count = start[field];
                }

                    /* decrement one more time because buf_index
                        gets incremented later in the loop: */
                buf_index--;
                break;

            case 'F':
                    /* Where in the line do we start?   */
                start[field] = st_count;
                    /* how many chars in this field?    */
                len[field] = ++l_count;
                    /* bump the field count             */
                field++;
                y++;
                l_count = 0;
                st_count = buf_index + 1;
                _settextcolor(new_fg_color);
                _setbkcolor((long)mark_color);
                break;

            case 'Q':
            case '\r':
                    /* quit: get the rest of the line and put it in
                        one big field if need be: */
                start[field] = buf_index ? st_count : 0;
                len[field] = max_len - st_count;
                field++;
                l_count = 0;
                buf_index = max_len;
                break;
                
            default:
                    /* I use the space bar. But pressing any character
                        moves one position to the right. */
                l_count++;
                y++;
                _setbkcolor((long)new_bk_color);
                _settextcolor(new_fg_color);
                break;
        }

        _outtext(tmp);
        if (y > 80)
        {
            y = 1;
            x++;
        }
        buf_index++;
    } while(buf_index < max_len);

    _settextposition(4,1);
    for(x = 0;x < field - 1;x++)
        printf("\"%.*s\",",len[x],&buf[start[x]]);
    printf("\"%.*s\"\n",len[x],&buf[start[x]]);
    _setbkcolor((long)old_bk_color);
    _settextcolor(old_fg_color);
}


int check_file(char *fname)
{
    FILE *fp, *fopen();
    char in_buf[80];
    int x;

    if( (fp = fopen(fname,"r")) == NULL)
        return 0;
    fgets(in_buf,80,fp);    /* infile name */
    x = 0;
    while(fgets(in_buf,80,fp) != NULL)
    {
        sscanf(in_buf,"%d %d",&start[x],&len[x]);
        x++;
    }
    field = x;
    return x;
}
